#include <winsock2.h>
#include "iptools.h"
#include "strings.h"

IPTools::IPTools()
{	m_buffer=NULL;	}

IPTools::~IPTools()
{	if(m_buffer) delete []m_buffer;	}

bool IPTools::IsIP(const char* ip)
{
	return strcount(ip,'.')!=4;
}

//converts network-byte ip address to in_addr struct
in_addr IPTools::toina(ULONG ipaddr)
{
	in_addr ina;
	ina.S_un.S_addr = ipaddr;
	return ina;
}

//gets error descriptions//////////////////
char* IPTools::GetLastErrorDescConnect()
{	return GetErrorDescConnect(WSAGetLastError()); }

char* IPTools::GetLastErrorDescSend()
{	return GetErrorDescSend(WSAGetLastError()); }

char* IPTools::GetErrorDescConnect(int errorcode)
{
	if(m_buffer) delete[]m_buffer;
	static char error[256];
	switch(errorcode)
	{
    case WSANOTINITIALISED : strcpy(m_buffer,"A successful WSAStartup call must occur before using this function.");break;
    case WSAENETDOWN       : strcpy(m_buffer,"The network subsystem has failed.");break;
    case WSAEADDRINUSE     : strcpy(m_buffer,"The socket's local address is already in use and the socket was not marked to allow address reuse with SO_REUSEADDR.");break;
    case WSAEINTR          : strcpy(m_buffer,"The blocking Windows Socket 1.1 call was canceled through WSACancelBlockingCall.");break;
    case WSAEINPROGRESS    : strcpy(m_buffer,"A blocking Windows Sockets 1.1 call is in progress, or the service provider is still processing a callback function.");break;
    case WSAEALREADY       : strcpy(m_buffer,"A nonblocking connect call is in progress on the specified socket.");break;
    case WSAEADDRNOTAVAIL  : strcpy(m_buffer,"The remote address is not a valid address (such as ADDR_ANY).");break;
    case WSAEAFNOSUPPORT   : strcpy(m_buffer,"Addresses in the specified family cannot be used with this socket.");break;
    case WSAECONNREFUSED   : strcpy(m_buffer,"The attempt to connect was forcefully rejected.");break;
    case WSAEFAULT         : strcpy(m_buffer,"The name or the namelen parameter is not a valid part of the user address space, the namelen parameter is too small, or the name parameter contains incorrect address format for the associated address family.");break;
    case WSAEINVAL         : strcpy(m_buffer,"The parameter s is a listening socket.");break;
    case WSAEISCONN        : strcpy(m_buffer,"The socket is already connected (connection-oriented sockets only).");break;
    case WSAENETUNREACH    : strcpy(m_buffer,"The network cannot be reached from this host at this time.");break;
    case WSAENOBUFS        : strcpy(m_buffer,"No buffer space is available. The socket cannot be connected.");break;
    case WSAENOTSOCK       : strcpy(m_buffer,"The descriptor is not a socket.");break;
    case WSAETIMEDOUT      : strcpy(m_buffer,"Attempt to connect timed out without establishing a connection.");break;
    case WSAEWOULDBLOCK    : strcpy(m_buffer,"The socket is marked as nonblocking and the connection cannot be completed immediately.");break;
    case WSAEACCES         : strcpy(m_buffer,"Attempt to connect datagram socket to broadcast address failed because setsockopt option SO_BROADCAST is not enabled.");break;
	default				   : strcpy(m_buffer,"Unkown error.");
	}
	return error;
}

char* IPTools::GetErrorDescSend(int errorcode)
{
	if(m_buffer) delete[]m_buffer;
	switch(errorcode)
	{
	case WSANOTINITIALISED : strcpy(m_buffer,"A successful WSAStartup call must occur before using this function."); break;
	case WSAENETDOWN       : strcpy(m_buffer,"The network subsystem has failed."); break;
	case WSAEACCES         : strcpy(m_buffer,"The requested address is a broadcast address, but the appropriate flag was not set. Call setsockopt with the SO_BROADCAST parameter to allow the use of the broadcast address."); break;
	case WSAEINTR          : strcpy(m_buffer,"A blocking Windows Sockets 1.1 call was canceled through WSACancelBlockingCall."); break;
	case WSAEINPROGRESS    : strcpy(m_buffer,"A blocking Windows Sockets 1.1 call is in progress, or the service provider is still processing a callback function."); break;
	case WSAEFAULT		   : strcpy(m_buffer,"The buf parameter is not completely contained in a valid part of the user address space."); break;
	case WSAENETRESET      : strcpy(m_buffer,"The connection has been broken due to the keep-alive activity detecting a failure while the operation was in progress."); break;
	case WSAENOBUFS        : strcpy(m_buffer,"No buffer space is available."); break;
	case WSAENOTCONN       : strcpy(m_buffer,"The socket is not connected."); break;
	case WSAENOTSOCK       : strcpy(m_buffer,"The descriptor is not a socket."); break;
	case WSAEOPNOTSUPP     : strcpy(m_buffer,"MSG_OOB was specified, but the socket is not stream-style such as type SOCK_STREAM, OOB data is not supported in the communication domain associated with this socket, or the socket is unidirectional and supports only receive operations."); break;
	case WSAESHUTDOWN      : strcpy(m_buffer,"The socket has been shut down); break; it is not possible to send on a socket after shutdown has been invoked with how set to SD_SEND or SD_BOTH."); break;
	case WSAEWOULDBLOCK    : strcpy(m_buffer,"The socket is marked as nonblocking and the requested operation would block."); break;
	case WSAEMSGSIZE       : strcpy(m_buffer,"The socket is message oriented, and the message is larger than the maximum supported by the underlying transport."); break;
	case WSAEHOSTUNREACH   : strcpy(m_buffer,"The remote host cannot be reached from this host at this time."); break;
	case WSAEINVAL         : strcpy(m_buffer,"The socket has not been bound with bind, or an unknown flag was specified, or MSG_OOB was specified for a socket with SO_OOBINLINE enabled."); break;
	case WSAECONNABORTED   : strcpy(m_buffer,"The virtual circuit was terminated due to a time-out or other failure. The application should close the socket as it is no longer usable."); break;
	case WSAECONNRESET     : strcpy(m_buffer,"The virtual circuit was reset by the remote side executing a hard or abortive close. For UPD sockets, the remote host was unable to deliver a previously sent UDP datagram and responded with a 'Port Unreachable' ICMP packet."); break;
	case WSAETIMEDOUT      : strcpy(m_buffer,"The connection has been dropped, because of a network failure or because the system on the other end went down without notice."); break;
	default				   : strcpy(m_buffer,"Unkown error.");
	}
	return m_buffer;
}

ULONG IPTools::GetHostIP(const char* domain)
{
	HOSTENT *phostent;
	if(!domain) return NULL;
	if(!(phostent=gethostbyname(domain))) return NULL;

	return *(ULONG*)(phostent->h_addr_list[0]);
}

ULONG* IPTools::GetHostIPList(const char* domain)
{
	HOSTENT *phostent;
	if(!domain) return NULL;
	if(!(phostent=gethostbyname(domain))) return NULL;

	return (ULONG*)(phostent->h_addr_list[0]);
}

char* IPTools::GetHostName(ULONG ipaddr)
{
	HOSTENT* phostent;

	if(m_buffer) delete[]m_buffer;
	
	if(!ipaddr)
		m_buffer = strnew(inet_ntoa(toina(ipaddr)));
	else if(ipaddr == 0x0100007f)
		m_buffer = strnew("localhost");
	else if(phostent=gethostbyaddr( (char *) &ipaddr, sizeof(ipaddr), AF_INET))
		m_buffer = strnew(phostent->h_name);
	else
		m_buffer = strnew(inet_ntoa(toina(ipaddr)));
	return m_buffer;
}

char* IPTools::GetWhois(const char* domain)
{
	if(!domain) return NULL;

	char* buffer;

	//create socket
	SOCKET sock;
	sockaddr_in sa;

	char whois_ipstr[] = "216.168.224.69";
	in_addr whois_ip = toina(inet_addr(whois_ipstr));
	USHORT whois_port = 43;

	sa.sin_family = AF_INET;
	sa.sin_port   = htons(whois_port);
	sa.sin_addr	  = whois_ip;

	if( !(sock=socket(AF_INET, SOCK_STREAM, 0)) )
	{
		PostError("Unable to create socket.");
		return NULL;
	}
	if( connect(sock, (sockaddr*)&sa, sizeof(sa))==SOCKET_ERROR )
	{
		PostError(GetLastErrorDescConnect());
		return NULL;
	}

	buffer=new char[strlen(domain)+3]; //crlf+null
	strcpy(buffer,domain);
	strcpy(buffer+strlen(domain),"\r\n");

	int n = strlen(buffer);

	if(send(sock, buffer, strlen(buffer), 0)==SOCKET_ERROR )
	{
		PostError(GetLastErrorDescSend());
		return NULL;
	}
	
	delete []buffer;

	//begin receiving
	const int PACKETSIZE = 1024;
	const int BUFFERSIZE = 1024;

	char packet[PACKETSIZE];
	int  packetlen;
	
	buffer=new char[BUFFERSIZE];
	ULONG bufferlen = 0;			//length of buffer
	ULONG bufferspc = BUFFERSIZE;	//amount of space allocated
	char* newbuffer;

	packet[256]=0;
	while( (packetlen=recv(sock, packet, PACKETSIZE, 0)) >0)
	{
		if( (bufferlen+packetlen)<bufferspc )	//can't be eq to...allow for null term
			memcpy(buffer+bufferlen, packet, packetlen);
		else
		{
			bufferspc=bufferlen+packetlen+1;
			newbuffer=new char[bufferspc];
			memcpy(newbuffer,buffer,bufferlen);
			memcpy(newbuffer+bufferlen,packet,packetlen);
			if(buffer) delete buffer;
			buffer=newbuffer;
		}
		bufferlen+=packetlen;
	}
	buffer[bufferlen]=0;
	closesocket(sock);

	m_buffer=strreplace(buffer,"\n","\r\n");
	delete []buffer;
	return m_buffer;
}

void IPTools::PostError(const char* error)
{
	if(m_error) delete []m_error;
	m_error=strnew(error);
}

char* IPTools::GetError()
{
	m_buffer=strnew(m_error);
	return m_buffer;
}