#include "strings.h"

char* strnew(const char* src)
{
	//creates a duplicate of src
	if(!src) return NULL;
	char* dest = new char[strlen(src)+1];
	strcpy(dest, src);
	return dest;
}

char* strmid(const char* src, const char* stop)
{
	//reads a string until it reaches stop
	if(!src || !stop) return NULL;
	char* pStop = strstr(src, stop);
	if(!pStop) return NULL;
	int len = pStop-src;		
	char* dest = new char[len+1];
	strncpy(dest,src,len);	//copy buffer
	dest[len]=0;			//terminate	
	return dest;
}

char* strreplace(const char* src, const char* find, const char* replace)
{
	//replaces all strings in a string to another string, creates new
	if(!src) return NULL;

	//count number of times found in string
	int i=0;
	int lenfind=strlen(find);
	const char* pLoc=src;
	while(pLoc=strstr(pLoc, find))
	{
		pLoc+=lenfind;
		i++;
	}

	//calculate buffer size necessary: lensrc - i*lenfind + i*lenreplace
	int lenrep=strlen(replace);
	char* dest=new char[strlen(src)+i*(lenrep-lenfind)+1];
	const char* pBeg=src;		//ptr to copy beg on src
	const char* pEnd;			//ptr to copy end on src
	char* pDest=dest;	//ptr to copy dest
	while(pEnd=strstr(pBeg, find))
	{
		strncpy(pDest, pBeg, pEnd-pBeg);	//copy characters before findchar
		strncpy(pDest+=pEnd-pBeg,replace,strlen(replace));	//don't call strcpy() because don't wall null char
		pDest+=lenrep;		
		pBeg=pEnd+lenfind;
	}
	strcpy(pDest,pBeg);

	return dest;
}

char* strreplace(char* src, const char find, const char replace)
{
	//replaces all characters in a string to another character
	if(!src) return NULL;

	for(unsigned i=0;i<strlen(src);i++)
		if(src[i]==find)
			src[i]=replace;
	return src;
}

char* strtrim(char* src)
{
	if(!src) return NULL;
	if(!strlen(src)) return src;

	unsigned iBeg = 0;
	unsigned iEnd = strlen(src)-1;

	//find first char
	while(iswspace(src[iBeg]) && iBeg<strlen(src))
		iBeg++;

	//find last char
	while(iswspace(src[iEnd]) && iEnd>0)
		iEnd--;

	unsigned i,j;

	j=0;	//dest idx
	for(i=iBeg; i<=iEnd; i++)
		src[j++]=src[i];
	src[j]=0;
	
	return src;
}

char* ReadLine(const char** ppStart)
{
	//finds crlf, copies line to new buffer, moves ptr to beginning of next line
	//sets ptr to null if last line
	int len;
	char* buf;

	if(*ppStart)
	{
		char* pEOL = strstr(*ppStart, "\r\n"); //find EOL

		if(!pEOL)		//last line
		{
			buf=new char[strlen(*ppStart)+1];
			strcpy(buf,*ppStart);
			*ppStart=NULL;
		}
		else
		{
			len=pEOL-*ppStart;
			buf = new char[len+1];
			strncpy(buf,*ppStart,len);
			buf[len]=0;
			*ppStart=pEOL+2;
		}
		return buf;
	}
	return NULL;
}

char* strjoin(const char* str1, const char* str2)
{
	//catenates both strings, and creates a new string
	char* str;
	if(!str1 && !str2) return NULL;
	if(!str1) str=strnew(str2);
	else if(!str2) str=strnew(str1);
	else
	{
		str=new char[strlen(str1)+strlen(str2)+1];
		strcpy(str,str1);
		strcat(str,str2);
	}
	return str;
}

char* strjoin(int nStrings, ...)
{
	int n=0;
	char* str=NULL;
	char* arg=NULL;
	
	va_list marker;
	va_start(marker, nStrings);
	arg=va_arg(marker, char*);
	while(n<nStrings)
	{
		str=strappend(str,arg);
		arg=va_arg(marker, char*);
		n++;
	}
	va_end(marker);
	return str;
}

char* strappend(char* str1, const char* str2)
{
	char* str;
	if(!str1 && !str2) return NULL;
	if(!str1) str=strnew(str2);
	else if(!str2) str=strnew(str1);
	else
	{
		str=new char[strlen(str1)+strlen(str2)+1];
		strcpy(str,str1);
		strcat(str,str2);
		if(str1) delete []str1;
	}
	return str;
}

char* strappend(char* str1, int nStrings, ...)
{
	int n=0;
	char* str=NULL;
	char* arg=NULL;
	
	va_list marker;
	va_start(marker, nStrings);
	str=strnew(str1);
	arg=va_arg(marker, char*);
	while(n++<nStrings)
	{
		str=strappend(str,arg);
		arg=va_arg(marker, char*);
	}
	va_end(marker);
	return str;
}

UINT strcount(const char* src, const char* find)
{
	//counts the number of occurences of a substring
	if(!src) return NULL;

	int i=0;
	int lenfind=strlen(find);
	const char* pLoc=src;
	while(pLoc=strstr(pLoc, find))
	{
		pLoc+=lenfind;
		i++;
	}
	return i;
}

UINT strcount(const char* src, char find)
{
	//counts the number of occurences of a character
	//replaces all characters in a string to another character
	if(!src) return NULL;

	UINT n=0;
	for(unsigned i=0;i<strlen(src);i++)
		if(src[i]==find)
			n++;
	return n;
}